body {
	color:			#000000;
	background-color:	#FFFFFF;
}

a:link {
	color:			#FF0000;
	text-decoration:	underline;
}

a:active {
	color:			#FF0000;
	text-decoration:	underline;
}

a:visited {
	color:			#000000;
	text-decoration:	underline;
}

.warning_text {
	color:			#FF0000;
	font-weight:		bold;
	text-align:		center;
	margin:			30px;
}
